package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.strike;

import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.util.IteratorUtil;
import org.bukkit.Bukkit;
import org.bukkit.Location;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.lang.reflect.Type;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class CordLeak {
    private static final Path saveFile = Paths.get(Main.instance().getDataFolder().getAbsolutePath(), "cordleak.json");
    private static final Map<UUID, Location> locations = new HashMap<>();

    static {
        load();
    }

    private record LocationDto(String world, double x, double y, double z, float yaw, float pitch) {
        static LocationDto from(Location loc) {
            return new LocationDto(loc.getWorld().getName(), loc.getX(), loc.getY(), loc.getZ(), loc.getYaw(), loc.getPitch());
        }

        Location toLocation() {
            return new Location(Bukkit.getWorld(this.world), this.x, this.y, this.z, this.yaw, this.pitch);
        }
    }

    public static void update() {
        IteratorUtil.onlinePlayers(player -> locations.put(player.getUniqueId(), player.getLocation()));
        Bukkit.getScheduler().runTaskAsynchronously(Main.instance(), CordLeak::save);
    }

    private static void load() {
        if (!Files.exists(saveFile)) return;
        try (Reader reader = Files.newBufferedReader(saveFile)) {
            Type type = new TypeToken<Map<String, LocationDto>>() {}.getType();
            Map<String, LocationDto> raw = new Gson().fromJson(reader, type);
            for (Map.Entry<String, LocationDto> entry : raw.entrySet()) {
                locations.put(UUID.fromString(entry.getKey()), entry.getValue().toLocation());
            }
        } catch (IOException e) {
            Main.logger().warning("Failed to load CordLeak data: " + e.getMessage());
        }
    }

    private static void save() {
        try {
            Files.createDirectories(saveFile.getParent());
            Map<String, LocationDto> raw = new HashMap<>();
            for (Map.Entry<UUID, Location> entry : locations.entrySet()) {
                raw.put(entry.getKey().toString(), LocationDto.from(entry.getValue()));
            }
            try (Writer writer = Files.newBufferedWriter(saveFile)) {
                new Gson().toJson(raw, writer);
            }
        } catch (IOException e) {
            Main.logger().warning("Failed to save CordLeak data: " + e.getMessage());
        }
    }

    public static Location getLastKnownLocation(UUID uuid) {
        return locations.get(uuid);
    }
}
