package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.strike;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.api.server.HttpServer;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.util.IteratorUtil;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.Teams;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.VaroTeam;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.OfflinePlayer;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

public class Strike extends Appliance {
    public Strike() {
        Bukkit.getScheduler().scheduleSyncRepeatingTask(
            Main.instance(),
            CordLeak::update,
            Ticks.TICKS_PER_SECOND,
            Ticks.TICKS_PER_SECOND * 3
        );
    }

    public void leakCoordinates() {
    }

    @Override
    public void httpApi(HttpServer.ApiBuilder apiBuilder) {
        record StrikeInfo(List<UUID> users, int totalWeight) {
        }
        apiBuilder.post("update", StrikeInfo.class, (data, request) -> {
            Main.instance().getLogger().info(String.format(
                "API Triggered Strike-Profile update for %s",
                data.users.stream().map(UUID::toString).collect(Collectors.joining(", ")))
            );

            VaroTeam select = null;
            for(UUID uuid : data.users) {
                select = this.queryAppliance(Teams.class).getTeamFromPlayer(uuid);
                if(select != null) break;
            }
            Objects.requireNonNull(select);
            VaroTeam team = select;

            switch(data.totalWeight) {
                case 1 -> {
                    Main.logger().info(String.format("Cord leak for Team %s", team.name));
                    ComponentBuilder<TextComponent, TextComponent.Builder> text = Component.text()
                        .append(
                            Component.text(
                                String.format("Das Team %s hat einen Strike erhalten. Daher werden folgende Koordinaten des Teams veröffentlicht:", team.name),
                                NamedTextColor.RED
                            )
                        );

                    data.users.forEach(uuid -> {
                        OfflinePlayer player = Bukkit.getOfflinePlayer(uuid);
                        Location cords = CordLeak.getLastKnownLocation(uuid);
                        if(cords == null) return;

                        text.appendNewline();
                        text.append(Component.text(Objects.requireNonNull(player.getName()),  NamedTextColor.DARK_AQUA));
                        text.append(Component.text(": ", NamedTextColor.GRAY));
                        text.append(Component.text(
                            String.format(
                                "(%s) %d, %d, %d",
                                cords.getWorld().getName(),
                                cords.getBlockX(),
                                cords.getBlockY(),
                                cords.getBlockZ()
                            ),
                            NamedTextColor.AQUA)
                        );
                    });

                    IteratorUtil.onlinePlayers(player -> player.sendMessage(text));
                }

                case 2 -> team.members.forEach(member -> {
                    Optional<Player> player = Optional.ofNullable(Bukkit.getPlayer(member.player.getUniqueId()));
                    player.ifPresentOrElse(
                        p -> p.getInventory().clear(),
                        () -> InvClear.add(member.player.getUniqueId())
                    );
                });
                case 3 -> team.members.forEach(member -> member.player.banPlayer("projektausschluss"));
            }

            return HttpServer.nothing;
        });
    }

    public void checkJoin(@NotNull Player player) {
        if(InvClear.contains(player.getUniqueId())) {
            player.getInventory().clear();
            InvClear.remove(player.getUniqueId());
        }
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new OnStrikeActionListener());
    }
}
